@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Draft201909; # git description: v0.005-17-ge93745f
+package JSON::Schema::Draft201909; # git description: v0.006-10-gc931ae6
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema data validation structure specification
 
-our $VERSION = '0.006';
+our $VERSION = '0.007';
 
 no if "$]" >= 5.031009, feature => 'indirect';
 use feature 'fc';
@@ -158,8 +158,7 @@ sub evaluate {
 
     abort($state, 'unable to find resource %s', $schema_reference) if not defined $schema;
 
-    $state->{canonical_schema_uri} = $canonical_uri->clone->fragment(undef);
-    $state->{schema_path} = $canonical_uri->fragment // '';
+    $state->{canonical_schema_uri} = $canonical_uri->clone->fragment($canonical_uri->fragment);
 
     $result = $self->_eval($data, $schema, $state);
   }
@@ -195,7 +194,7 @@ sub _eval {
   my $schema_type = $self->_get_type($schema);
   return $schema || E($state, 'subschema is false') if $schema_type eq 'boolean';
 
-  abort($state, 'unrecognized schema type "%s"', $schema_type) if $schema_type ne 'object';
+  abort($state, 'invalid schema type: %s', $schema_type) if $schema_type ne 'object';
 
   my $result = 1;
 
@@ -299,7 +298,7 @@ sub _fetch_and_eval_ref_uri {
   return $self->_eval($data, $subschema,
     +{ %$state,
       traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path}.'/'.$state->{keyword},
-      canonical_schema_uri => $canonical_uri, # note: not canonical yet until $id is processed
+      canonical_schema_uri => $canonical_uri, # note: maybe not canonical yet until $id is processed
       schema_path => '',
     });
 }
@@ -349,7 +348,6 @@ sub _eval_keyword_comment {
 sub _eval_keyword_defs {
   my ($self, $data, $schema, $state) = @_;
 
-  my $type = $self->_get_type($schema->{'$defs'});
   assert_keyword_type($state, $schema, 'object');
 
   # we do nothing directly with this keyword, including not collecting its value for annotations.
@@ -838,7 +836,7 @@ sub _eval_keyword_patternProperties {
     }
     catch {
       abort({ %$state,
-        schema_path_rest => jsonp($state->{schema_path}, 'patternProperties', $property_pattern) },
+        _schema_path_rest => jsonp($state->{schema_path}, 'patternProperties', $property_pattern) },
       $@);
     };
     foreach my $property (sort @matched_properties) {
@@ -928,7 +926,7 @@ has _format_validations => (
       eval { Time::Moment->from_string($_[0]) } ? 1 : 0,
     };
     my $is_email = sub {
-      eval { require Email::Address::XS; 1 } or return 1;
+      eval { require Email::Address::XS; Email::Address::XS->VERSION(1.01); 1 } or return 1;
       Email::Address::XS->parse($_[0])->is_valid;
     };
     my $is_hostname = sub {
@@ -1235,7 +1233,7 @@ sub _fetch_schema_from_uri {
     my $base = $uri->clone->fragment(undef);
     if (my $resource = $self->_get_or_load_resource($base)) {
       $subschema = $resource->{document}->get($resource->{path}.$fragment);
-      $canonical_uri = $uri;
+      $canonical_uri = $resource->{canonical_uri}->clone->fragment($uri->fragment);
     }
   }
   else {
@@ -1267,15 +1265,15 @@ sub E {
   my ($state, $error_string, @args) = @_;
 
   # sometimes the keyword shouldn't be at the very end of the schema path
-  my $schema_path_rest = $state->{schema_path_rest}
+  my $schema_path = delete $state->{_schema_path_rest}
     // $state->{schema_path}.($state->{keyword} ? '/'.$state->{keyword} : '');
 
   push @{$state->{errors}}, JSON::Schema::Draft201909::Error->new(
     instance_location => $state->{data_path},
-    keyword_location => $state->{traversed_schema_path}.$schema_path_rest,
+    keyword_location => $state->{traversed_schema_path}.$schema_path,
     !"$state->{canonical_schema_uri}" ? () : ( absolute_keyword_location => do {
       my $uri = $state->{canonical_schema_uri}->clone;
-      $uri->fragment(($uri->fragment//'').$schema_path_rest) if $schema_path_rest;
+      $uri->fragment(($uri->fragment//'').$schema_path) if $schema_path;
       $uri;
     } ),
     error => @args ? sprintf($error_string, @args) : $error_string,
@@ -1316,7 +1314,7 @@ JSON::Schema::Draft201909 - Validate data against a schema
 
 =head1 VERSION
 
-version 0.006
+version 0.007
 
 =head1 SYNOPSIS
 
